/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkDummyMBReader.cxx,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#include "vtkDummyMBReader.h"

#include "vtkUnstructuredGrid.h"
#include "vtkStructuredGrid.h"
#include "vtkInformation.h"
#include "vtkInformationVector.h"
#include "vtkObjectFactory.h"
#include "vtkErrorCode.h"
#include "vtkCellType.h"
#include "vtkCellArray.h"
#include "vtkGarbageCollector.h"
#include "vtkAppendPolyData.h"
#include "vtkPointData.h"
#include "vtkCellData.h"

#include "vtkUnsignedIntArray.h"
#include "vtkDoubleArray.h"
#include "vtkSmartPointer.h"

#include <sys/stat.h>
#include <vtksys/SystemTools.hxx>

vtkStandardNewMacro(vtkDummyMBReader);


//----------------------------------------------------------------------------
vtkDummyMBReader::vtkDummyMBReader()
{
  this->FileName  = NULL;
  this->SetNumberOfInputPorts(0);

  this->rootMB_init=false;
  
  this->selector=-1;

}

//----------------------------------------------------------------------------
vtkDummyMBReader::~vtkDummyMBReader()
{
  if (this->FileName)
    {
    delete [] this->FileName;
    }
    this->rootMB=NULL;
}


//inits MB
int vtkDummyMBReader::InitMB(){

    this->rootMB=vtkSmartPointer<vtkMultiBlockDataSet>::New();   //level0
    this->rootMB->SetNumberOfBlocks(1);

    vtkSmartPointer<vtkMultiBlockDataSet> level1=vtkSmartPointer<vtkMultiBlockDataSet>::New();//level1
    level1->SetNumberOfBlocks(3);

    for(int iL2=0; iL2 < 3 ; iL2++){
        vtkSmartPointer<vtkMultiBlockDataSet> level2=vtkSmartPointer<vtkMultiBlockDataSet>::New();  //level2

        if (iL2==1){
            for( int iL3=0; iL3 < 20 ; iL3++){
              vtkSmartPointer<vtkDataObject>  grid = vtkSmartPointer<vtkDataObject>::New();
              level2->SetBlock(iL3, grid);
              
              //By default all  blocks haven't been loaded
              this->isLoaded.push_back(false);
              
            } // for iL2
        }//if iL1
        level1->SetBlock(iL2,level2);
    }//for iL1
    this->rootMB->SetBlock(0,level1);

    return 1;
}





void  vtkDummyMBReader::LoadUnload_Grid(int blockIndex){
     
  
              
  vtkMultiBlockDataSet *p_level0 = vtkMultiBlockDataSet::SafeDownCast( this->rootMB);  //pointer level 0, it is MSB and has 1 child
              
  vtkMultiBlockDataSet *p_level1= p_level0->vtkMultiBlockDataSet::SafeDownCast(p_level0->GetBlock(0))  ;   //pointer level 1, we know it is a MB and has 3 Childern (all MB)
              
  for(int iL2=0; iL2 < 3 ; iL2++){
                
    vtkMultiBlockDataSet *p_level2=vtkMultiBlockDataSet::SafeDownCast(p_level1->GetBlock(iL2)); //pointer level 2, we know it is a MB  and the one with index 1 has 5 Children (Structured/unstructured...)
                
    if (iL2==1){
      
          if ( !this->isLoaded.at( blockIndex ) ){
              cout<<"Block "<< blockIndex <<" is OFF. Turning ON"<<endl;
              this->isLoaded.at( blockIndex )=true;
              int iL3=blockIndex;
              vtkSmartPointer<vtkStructuredGrid> p_grid ;
              p_grid=ReadGrid(500 ,0.01, iL3 , iL3, iL3);
              p_level2->SetBlock(iL3,p_grid);               
          }
          else if ( this->isLoaded.at( blockIndex ) ){
              cout<<"Block "<< blockIndex <<" is ON. Turning OFF"<<endl;
              this->isLoaded.at( blockIndex )=false;
              int iL3=blockIndex;
              vtkSmartPointer<vtkDataObject> p_grid=vtkSmartPointer<vtkDataObject>::New() ;
              //vtkStructuredGrid *p_grid=vtkStructuredGrid::SafeDownCast( p_level2->GetBlock(iL3));
              //p_grid->Delete();
              
              p_level2->SetBlock(iL3,p_grid);
          }  
                               
    }  //for iL2
  }
  

vtkCompositeDataIterator* iter = this->rootMB->NewIterator();
for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
  {
  
   cout<<"Size"<<iter->GetActualMemorySize()<<endl;
  }
iter->Delete();
}// vtkDummyMBReader::LoadUnload_Grid




vtkSmartPointer<vtkStructuredGrid>  vtkDummyMBReader::ReadGrid(int n, double delta , double x , double y, double z){
  
    vtkSmartPointer<vtkStructuredGrid>  subGrid=vtkSmartPointer<vtkStructuredGrid>::New();
       
    //subGrid->Initialize();
    //subGrid->Allocate();

    //Nodes| points
 	  vtkSmartPointer<vtkPoints> points = vtkSmartPointer<vtkPoints>::New();
    subGrid->SetPoints(points);
    int pointCnt=0;
    vtkSmartPointer<vtkDoubleArray> valPoint=vtkSmartPointer<vtkDoubleArray>::New();
    valPoint->SetNumberOfTuples(n*n*n);
    valPoint->SetName("Node Index");
    for(int i =0 ; i < n ; i++ ){
       for(int j=0;  j < n ; j++ ){
          for(int k=0 ; k < n ; k++ ){
              points->InsertPoint (pointCnt,i*delta+x,j*delta+y,k*delta+z);
              //valPoint->InsertTuple1(pointCnt, pointCnt);
              pointCnt++;
          }
       }// for j 
    }// for i 
    vtkPointData* p_pointData = subGrid->GetPointData();
    p_pointData->AddArray(valPoint);
    
    
    subGrid->SetDimensions(n,n,n);
    // cells   
    /*vtkIdType cell[8];
    vtkSmartPointer<vtkDoubleArray> valCell=vtkSmartPointer<vtkDoubleArray>::New();
    valCell->SetNumberOfTuples((n-1)*(n-1)*(n-1));
    valCell->SetName("Cell Index");
    int cellCnt=0;
    
    for(int i =0 ; i < n-1 ; i++ ){
       for(int j=0;  j < n-1 ; j++ ){
          for(int k=0 ; k < n-1 ; k++ ){
              valCell->InsertTuple1(cellCnt,cellCnt);
              cellCnt++;
          }//for k 
       }// for j 
   }// for i 
   vtkCellData* p_cellData = subGrid->GetCellData();
   p_cellData->AddArray(valCell);
    */   
   return  subGrid; 
}




// get Block Number from GUI
void vtkDummyMBReader::TestProperty(int val){
  if ( val>-1 ){
     this->selector=val;
     this->Modified();
  }
}


//----------------------------------------------------------------------------
int vtkDummyMBReader::RequestData(
    vtkInformation *vtkNotUsed(request),
    vtkInformationVector **vtkNotUsed(inputVector),
   vtkInformationVector *outputVector)
{
    
  if (!this->rootMB_init){
       this->InitMB();
       this->rootMB_init=true;
  }

    
  if (this->rootMB_init  &&
      this->selector >-1 &&
      this->selector< this->isLoaded.size())
  {
          this->LoadUnload_Grid(this->selector);  

  }
  
  
  vtkInformation *outInfo = outputVector->GetInformationObject(0);
  vtkMultiBlockDataSet *output = vtkMultiBlockDataSet::SafeDownCast(outInfo->Get(vtkDataObject::DATA_OBJECT()));
  output->ShallowCopy(this->rootMB);
  return 1;
}


//----------------------------------------------------------------------------
void vtkDummyMBReader::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);

  os << indent << "File Name: "
    << (this->FileName ? this->FileName : "(none)") << "\n";
}
